#include "XYZDistributionProcessor.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/Track.h>
#include <EVENT/TrackerHit.h>
#include "IntersectionCalculator.h"

// Aida
//#ifdef MARLIN_USE_AIDA // the whole processor does not make sense without aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
// endif

#include <marlin/Global.h>

#include "TrackFitterFactory.h"
#include "TrackFitterBase.h"

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

XYZDistributionProcessor aXYZDistributionProcessor;
    
XYZDistributionProcessor::XYZDistributionProcessor(): Processor("XYZDistributionProcessor")
						    , _distributionXHisto(0) , _distributionYHisto(0), _distributionZHisto(0), _distributionXYHisto(0)
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACKERHIT,
				"InputHits" , 
				"Name of the input hits collection"  ,
				_inputColName ,
				std::string("TPCHits") ) ;
    // modify processor description
    _description = "XYZDistributionProcessor: Fill x, y, and z distribution of all hits into AIDA histgram" ;

  registerProcessorParameter( "HistogramMinValueX" , 
			      "Minimum value in x histogram (default: 0. mm)"  ,
			      _histogramMinValueX ,
			      float(0.) ) ;

  registerProcessorParameter( "HistogramMaxValueX" , 
			      "Maximal value in x histogram (default:  15. mm)"  ,
			      _histogramMaxValueX ,
			      float(15.) ) ;
  registerProcessorParameter( "HistogramMinValueY" , 
			      "Minimum value in y histogram (default: 0. mm)"  ,
			      _histogramMinValueY ,
			      float(0.) ) ;

  registerProcessorParameter( "HistogramMaxValueY" , 
			      "Maximal value in y histogram (default:  15. mm)"  ,
			      _histogramMaxValueY ,
			      float(15.) ) ;

  registerProcessorParameter( "HistogramMinValueZ" , 
			      "Minimum value in z histogram (default: 0.0 mm)"  ,
			      _histogramMinValueZ ,
			      float(0.0) ) ;

  registerProcessorParameter( "HistogramMaxValueZ" , 
			      "Maximal value in z histogram (default:  260.0 mm)"  ,
			      _histogramMaxValueZ ,
			      float(260.0) ) ;
}

XYZDistributionProcessor::~XYZDistributionProcessor()
{
}

void XYZDistributionProcessor::init()
{ 
	// usually a good idea to
	printParameters() ;

	// get a histograms from the AIDAProcessor
	_distributionXHisto =   AIDAProcessor::histogramFactory(this)->
	  createHistogram1D( "xDistributionHisto", 
			     "xDistributionHisto", 
			     100, _histogramMinValueX, _histogramMaxValueX);

	_distributionYHisto =   AIDAProcessor::histogramFactory(this)->
	  createHistogram1D( "yDistributionHisto", 
			     "yDistributionHisto", 
			     100, _histogramMinValueY, _histogramMaxValueY);

	_distributionZHisto =   AIDAProcessor::histogramFactory(this)->
	  createHistogram1D( "zDistributionHisto", 
			     "zDistributionHisto", 
			     100, _histogramMinValueZ, _histogramMaxValueZ);
	_distributionXYHisto =   AIDAProcessor::histogramFactory(this)->
	  createHistogram2D( "xyDistributionHisto", 
			     "xyDistributionHisto", 
			     900, _histogramMinValueX, _histogramMaxValueX,
			     900, _histogramMinValueY, _histogramMaxValueY);
    }

void XYZDistributionProcessor::processRunHeader( EVENT::LCRunHeader* run )
{
}
    
void XYZDistributionProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data

    // we need a LCCollectionVec to use iterators, as LCCollectio is not a std::vector
    LCCollectionVec* InputCollection=0;
    try{
	InputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	// no input data
	// just continue ...
	return;
    }


    // cast did not succeed
    if( InputCollection == 0 )
    {
	streamlog_out(ERROR2) << "InputCollection vector is 0" << std::endl;
	return;
    }

    for(LCCollectionVec::iterator it = InputCollection->begin() ; 
	it < InputCollection->end(); it ++ )
    {
	
	const double *pos = dynamic_cast<TrackerHit *>(*it)->getPosition();
	    
	_distributionXHisto->fill(pos[0]);
	_distributionYHisto->fill(pos[1]);
	_distributionZHisto->fill(pos[2]);
//	    if (pos[2] < 10)
	_distributionXYHisto->fill(pos[0],pos[1]);
	
    }
    
}


}// namespace 
