#ifndef SIMPLE_CHI_SQUARE_RESIDUALS_FUNCTION_H
#define SIMPLE_CHI_SQUARE_RESIDUALS_FUNCTION_H 1

#include <Minuit2/FCNBase.h>
#include <EVENT/Track.h>
#include <TrackFitterSimpleChiSquare.h>

namespace marlintpc
{

/**
 *  The Minuit2 fitter function to calculate the \f$\chi^2\f$ of the residuals. Derived from FCNBase.
 *  If noZFit is set in the fitter the squares of the z residuals are not added to the \f$\chi^2\f$.
 *  
 */

class SimpleChiSquareResidualsFunction : public ROOT::Minuit2::FCNBase {

 public:
    SimpleChiSquareResidualsFunction(const EVENT::Track * const t, TrackFitterSimpleChiSquare *f);
    ~SimpleChiSquareResidualsFunction() {}
    virtual double Up() const {return theErrorDef;}
    virtual double operator()(const std::vector<double>&) const;
    void SetErrorDef(double def) {theErrorDef = def;}

 private:

    const EVENT::Track * const track;
    TrackFitterSimpleChiSquare *fitter;
    double theErrorDef;	
};

}//namespace marlintpc

#endif //CHI_SQUARE_RESIDUALS_FUNCTION_H
