#include "ADCElectronicsHandler.h"

#include <utility>
#include <map>
#include <iostream>
#include <string.h>

//lccd
#include <lccd.h>
#include <lccd/LCConditionsMgr.hh>

//MarliNTPC
#include "TPCCondDataExeptions.h"

namespace marlintpc
{

    ADCElectronicsHandler::ADCElectronicsHandler (std::string ConditionsName, std::string ColName )
    {
        if ( ! (marlin::ConditionsProcessor::registerChangeListener( this, ColName ) ) )
        {
            m_out(ERROR)<<"ERROR: ADCElectronicsHandler cannot register collection "
                <<ColName<<" in ConditionsProcessor"<<std::endl;
            throw DataNotAvailableException("ADCElectronicsHandler: cannot register ADC electronics parameters collection");
        }
        else
        {
            lccd::LCConditionsMgr::instance()->getHandler( ColName )->registerChangeListener( this );
        }
        _conditionsName=ConditionsName;
        _genericADCElectronicsParametersMap.clear();
    }

    ADCElectronicsHandler::~ADCElectronicsHandler()
    {
        // empty
    }

    void ADCElectronicsHandler::conditionsChanged( EVENT::LCCollection* col )
    {
        _genericADCElectronicsParametersMap.clear();
        int nEntries = col->getNumberOfElements();

        if (_conditionsName == "GenericADCElectronicsParameters" )
        {
            _genericADCElectronicsParametersMap.clear();
            tpcconddata::GenericADCElectronicsParameters* Entry;                std::pair<GenericADCElectronicsParametersMap_t::iterator,bool> iInsertReturn;
            m_out(MESSAGE)<<"ADCElectronicsHandler: update conditions data  ("<<nEntries<<" in collection)"<<std::endl;
            for (int iEntry = 0; iEntry < nEntries; ++iEntry)
            {
                Entry = new tpcconddata::GenericADCElectronicsParameters(
                    dynamic_cast<LCGenericObject*>( col->getElementAt( iEntry ) ));
                iInsertReturn = _genericADCElectronicsParametersMap.insert(std::make_pair(Entry->getReadoutGroup(), Entry));
            }
            if ( !iInsertReturn.second )
            {
                m_out(WARNING)<<"ADCElectronicsHandler: could not store GenericADCElectronicsParameters for readout group: ("<<Entry->getReadoutGroup()<<") in the conditions map because there was already a value stored for this readout group number."<<std::endl;
            }
        }
        else
        {
            throw tpcconddata::CondDataWrongObjectException(std::string("Conditions Object " + _conditionsName + " not implemented in Handler."));
        }
    }

    int ADCElectronicsHandler::getPolarity(int readoutGroupID)
    {
        if (_conditionsName == "GenericADCElectronicsParameters" )
        {
            GenericADCElectronicsParametersMap_t::iterator Entry = _genericADCElectronicsParametersMap.find(readoutGroupID);
            if ( Entry == _genericADCElectronicsParametersMap.end() )
            {
                std::stringstream out;
                out << "Conditions data not available for readout group id "<<readoutGroupID<<".";
                throw tpcconddata::CondDataNotAvailableException(out.str());
            }
            else
            {
                return (Entry->second)->getPolarity();
            }
        }
        else
        {
            std::stringstream out;
            out << "Conditions data Polarity not available for " + _conditionsName + " object.";
            throw tpcconddata::CondDataNotAvailableException(out.str());
        }
    }

    int ADCElectronicsHandler::getMaxADCValue(int readoutGroupID)
    {
        if (_conditionsName == "GenericADCElectronicsParameters" )
        {
            GenericADCElectronicsParametersMap_t::iterator Entry = _genericADCElectronicsParametersMap.find(readoutGroupID);
            if ( Entry == _genericADCElectronicsParametersMap.end() )
            {
                std::stringstream out;
                out << "Conditions data not available for readout group id "<<readoutGroupID<<".";
                throw tpcconddata::CondDataNotAvailableException(out.str());
            }
            else
            {
                return (Entry->second)->getMaxADCValue();
            }
        }
        else
        {
            std::stringstream out;
            out << "Conditions data MaxADCValue not available for " + _conditionsName + " object.";
            throw tpcconddata::CondDataNotAvailableException(out.str());
        }
    }

    float ADCElectronicsHandler::getFrequency(int readoutGroupID)
    {
        if (_conditionsName == "GenericADCElectronicsParameters" )
        {
            GenericADCElectronicsParametersMap_t::iterator Entry = _genericADCElectronicsParametersMap.find(readoutGroupID);
            if ( Entry == _genericADCElectronicsParametersMap.end() )
            {
                std::stringstream out;
                out << "Conditions data not available for readout group id "<<readoutGroupID<<".";
                throw tpcconddata::CondDataNotAvailableException(out.str());
            }
            else
            {
                //return ((*(Entry)).second)->getFrequency();
                return (Entry->second)->getFrequency();
            }
        }
        else
        {
            std::stringstream out;
            out << "Conditions data Frequency not available for " + _conditionsName + " object.";
            throw tpcconddata::CondDataNotAvailableException(out.str());
        }
    }

}
