#include "PedestalHandler.h"

//lccd
#include <lccd.h>
#include <lccd/LCConditionsMgr.hh>

namespace marlintpc
{

    PedestalHandler::PedestalHandler (std::string ColName )
    {
        if ( ! (marlin::ConditionsProcessor::registerChangeListener( this, ColName ) ) )
        {
            m_out(ERROR)<<"ERROR: PedestalHandler cannot register collection "
                <<ColName<<" in ConditionsProcessor"<<std::endl;
            throw DataNotAvailableException("PedestalHandler: cannot register pedestal collection");
        }
        else
        {
            lccd::LCConditionsMgr::instance()->getHandler( ColName )->registerChangeListener( this );
        }
        _pedestalPadMap.clear();
        _pedestalModuleMap.clear();
    }

    PedestalHandler::~PedestalHandler()
    {
        // empty
    }

    void PedestalHandler::conditionsChanged( EVENT::LCCollection* col )
    {
        _pedestalPadMap.clear();
        _pedestalModuleMap.clear();

        int nEntries = col->getNumberOfElements();
        ModuleChannelPair_t channelID;
        tpcconddata::Pedestal*          Entry = 0;
        std::pair<PedestalPadMap_t::iterator,bool> iInsertReturn;
        PedestalModuleMap_t::iterator   iModuleMapElement;
        PedestalPadMap_t*               tmpPedestalMapInstance;

        m_out(MESSAGE)<<"PedestalHandler: update pedestals  ("<<nEntries<<" in collection)"<<std::endl;

        // fill LCCD collection into the map(map(pedestal)) structure
        for (int iEntry = 0; iEntry < nEntries; ++iEntry)
        {
            Entry = new tpcconddata::Pedestal( dynamic_cast<LCGenericObject*>( col->getElementAt( iEntry ) ) );
            channelID.first  = (Entry->getHardwareID()).first;
            channelID.second = (Entry->getHardwareID()).second;
            iModuleMapElement = (_pedestalModuleMap.find(channelID.second));
            if ( iModuleMapElement == _pedestalModuleMap.end() )
            {
                tmpPedestalMapInstance = new PedestalPadMap_t;
                _pedestalModuleMap.insert( std::make_pair(channelID.second, tmpPedestalMapInstance ) );
            }
            else
            {
                tmpPedestalMapInstance = (*iModuleMapElement).second;
            }
            iInsertReturn = tmpPedestalMapInstance->insert(std::make_pair(channelID.first,Entry));
            if ( !iInsertReturn.second )
            {
                m_out(WARNING)<<"PedestalHandler: could not store Pedestal for hardwareID: ("<<channelID.first<<","<<channelID.second<<") in the pedestal map because there was already a value stored for this channel"<<std::endl;
            }
        }                        //loop over all entries in the LCCollectionVec
    }

    tpcconddata::Pedestal* PedestalHandler::getPedestal( int channelID, int readoutGroup )
    {
        if ( _pedestalModuleMap.find(readoutGroup) !=  _pedestalModuleMap.end() )
        {
            if ( ((*(_pedestalModuleMap.find(readoutGroup))).second)->find(channelID) !=  ((*(_pedestalModuleMap.find(readoutGroup))).second)->end() )
            {
                return (*(((*(_pedestalModuleMap.find(readoutGroup))).second)->find(channelID))).second;
            }
            else
            {
                return 0;
            }
        }
        else
        {
            return 0;
        }
    }

    tpcconddata::Pedestal* PedestalHandler::getPedestal( ModuleChannelPair_t hardwareID )
    {
        return this->getPedestal( hardwareID.first, hardwareID.second );
    }

    float PedestalHandler::getPedestalValue( ModuleChannelPair_t hardwareID )
    {
        tpcconddata::Pedestal* Entry = getPedestal( hardwareID );
        if (Entry == 0 )
        {
            return 0.0;
        }
        else
        {
            return Entry->getPedestalValue();
        }
    }

    float PedestalHandler::getPedestalValue( int channelID, int readoutGroup )
    {
        ModuleChannelPair_t hardwareID( channelID, readoutGroup );
        return this->getPedestalValue( hardwareID );
    }

    float PedestalHandler::getPedestalWidth( ModuleChannelPair_t hardwareID )
    {
        tpcconddata::Pedestal* Entry = getPedestal( hardwareID );
        if (Entry == NULL )
        {
            return -1.0;
        }
        else
        {
            return Entry->getPedestalWidth();
        }
    }

    float PedestalHandler::getPedestalWidth( int channelID, int readoutGroup )
    {
        ModuleChannelPair_t hardwareID( channelID, readoutGroup );
        return this->getPedestalWidth( hardwareID );
    }

}
