#include "SimpleChiSquareResidualsFunction.h"
#include <cmath>
#include <streamlog/streamlog.h>

namespace marlintpc
{
SimpleChiSquareResidualsFunction::SimpleChiSquareResidualsFunction(const EVENT::Track * const t, 
								   TrackFitterSimpleChiSquare *f)
 : track( t) , fitter(f), theErrorDef(1.)
{
  streamlog_out(DEBUG2) << "SimpleChiSquareResidualsFunction constructor " << std::endl;
}
 
double SimpleChiSquareResidualsFunction::operator()(const std::vector<double>&trackParams) const
{
    double chiSquare=0;
    IMPL::TrackImpl trackWithTestParameters;
    trackWithTestParameters.setD0(trackParams[0]);
    trackWithTestParameters.setPhi(trackParams[1]);
    trackWithTestParameters.setOmega(trackParams[2]);
    trackWithTestParameters.setTanLambda(trackParams[3]);
    trackWithTestParameters.setZ0(trackParams[4]);

    streamlog_out(DEBUG1) << "SimpleChiSquareResidualsFunction: "
			  << " ; d0: "<< trackParams[0]
			  << " ; phi: "<< trackParams[1]
			  << " ; omega: "<< trackParams[2]
			  << " ; tamLambda: "<< trackParams[3]
			  << " ; z0: "<< trackParams[4]
			  << std::endl;

   
    
    for (TrackerHitVec::const_iterator hitIter = track->getTrackerHits().begin() ;
	 hitIter < track->getTrackerHits().end() ; hitIter++ )
    {
	EVENT::DoubleVec residuals=fitter->calculateResiduals(*hitIter, &trackWithTestParameters);

	double sigma_trans = sqrt( fitter->getTransDefocussing()*fitter->getTransDefocussing() 
				   + fitter->getTransDiffusionCoef() * fitter->getTransDiffusionCoef()
				   * (*hitIter)->getPosition()[2]);

	// FIXME: calculate real chiSquare taking into account the errors
	chiSquare += residuals[0]*residuals[0]/(sigma_trans*sigma_trans);

	if ( !fitter->getNoZFitFlag() ) // z direction only if not disabled
	{
	    
	    double sigma_long = sqrt( fitter->getLongDefocussing()*fitter->getLongDefocussing() 
				      + fitter->getLongDiffusionCoef() * fitter->getLongDiffusionCoef()
				      * (*hitIter)->getPosition()[2]);
	    streamlog_out(DEBUG0) << "SimpleChiSquareResidualsFunction: sigmalong"  << sigma_long
				  << " ; sigmatrans "<< sigma_trans << std::endl;
	    streamlog_out(DEBUG0) << "SimpleChiSquareResidualsFunction: resi"  << residuals[0]
				  << " ; "<< residuals[1] << std::endl;
	    
	    chiSquare += residuals[1]*residuals[1]/(sigma_long*sigma_long) ;
	}
    }

    streamlog_out(DEBUG1) << "SimpleChiSquareResidualsFunction: chiSquare is "<<chiSquare<< std::endl;
    return chiSquare;
}


}//namespace marlintpc
