#ifndef CHARGESIGNAL_H
#define CHARGESIGNAL_H 1

// stl
#include <string>

// LCIO
#include <UTIL/LCFixedObject.h>

namespace marlintpc {

/** ChargeSignal : A parameterized signal on a pad
 *
 *    @author J. Abernathy (UVic)<br>
 */
class ChargeSignal : public UTIL::LCFixedObject<4, 0, 2> {

public:
	/** constructor
	 */
	ChargeSignal(int, int, int);


	/** gets the number of electrons
	 */
	int get_electron_count( void ) const;


	/** add an electron to the signal at a specified time
	 */
	void add_electron( const double );


	/** gets the unique id of this charge signal
	 */
	int get_charge_id( void ) const;


	/** gets the pad id of this signal
	 */
	int get_pad_id( void ) const;


	/** gets the module id of this charge signal
	 */
	int get_module_id( void ) const;


	/** gets the variance in the time
	 */
	double get_time_variance( void ) const;


	/** gets the mean time of the signal
	 */
	double get_mean_time( void ) const;


	/** returns the typename of the class
	 */
	const std::string getTypeName( void ) const;


	/** returns the type of data in the generic object
	 */
	const std::string getDataDescription( void ) const;

protected:

private:

	/** gets the number of electrons
	 */
	void set_electron_count( const int );


	/** sets the unique id of this charge signal
	 */
	void set_charge_id( const int );


	/** sets the pad id of this signal
	 */
	void set_pad_id( const int );


	/** sets the variance in the time
	 */
	void set_time_variance( const double );


	/** sets the module id of this charge signal
	 */
	void set_module_id( const int );


	/** sets the mean time of the signal
	 */
	void set_mean_time( const double );
};

} // namespace marlintpc

#endif // CHARGESIGNAL_H
