#include "lcio.h"
//
#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"
//
#include "IMPL/LCEventImpl.h" 
//#include "IMPL/LCRunHeaderImpl.h" 
#include "IMPL/LCCollectionVec.h"
#include "IMPL/LCTOOLS.h"
//#include "IMPL/TrackerRawDataImpl.h"
//
//#include "gearimpl/RectangularPadRowLayout.h"
#include "gear/TPCParameters.h"
#include "gear/TPCModule.h"
#include "gearxml/GearXML.h"
#include "gear/GearMgr.h"
#include "gear/GEAR.h"
//
#include "ADCChannelMapping.h"
//
//#include <iostream>
//#include <sstream>
//#include <string>
//#include <cstdio>
//#include <typeinfo>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

using namespace std ;
using namespace lcio ;
using namespace gear;
using namespace tpcconddata;

void printUsage()
{
  cout << "MappingGenerator mapping.txt gearfile.xml [outfile.slcio]" <<endl;
    cout <<" outfile name is optional, "
	 << "default is TPCChannelMapping.slcio"<< endl;
    cout << "Text file format: hwGroup hwChannel moduleID row padNrInRow" << endl;
}

int main(int argc, char **argv)
{

  if (argc!=3 && argc!=4 )
  {
    printUsage();
    return(1);
  }

  
  // read pad layout from gear
  GearXML gearXML( argv[2] ) ;
  GearMgr* gearMgr = gearXML.createGearMgr() ;
  TPCParameters const &tpcParams=gearMgr->getTPCParameters();
  
 
  // get outfile name
  string outFileName;
  if (argc==4)
    outFileName=argv[3];
  else
    outFileName="TPCChannelMapping.slcio";

  // open outfile			     
  LCWriter* lcWrt = LCFactory::getInstance()->createLCWriter() ;
    
  lcWrt->open( outFileName, LCIO::WRITE_NEW ) ;
  
// do we need a run header?
//  // create runheader and write it
//  // the information should come from a file
//  // or be given interactively in a future version
//  LCRunHeaderImpl* runHdr = new LCRunHeaderImpl ; 
//  int runNr=0;
//  runHdr->setRunNumber( runNr) ;
//  
//  string detName("TPC")  ;
//  runHdr->setDetectorName( detName ) ;
//  
//  stringstream description ; 
//  description << "Pad mapping for a TPC";
//  runHdr->setDescription( description.str()  ) ;
//  
//  string tpcName("TPC") ;
//  runHdr->addActiveSubdetector( tpcName ) ;
//  
//  lcWrt->writeRunHeader( runHdr ) ;

  // create new lcio event
  LCEventImpl*  evt = new LCEventImpl() ;
  
  //  evt->setRunNumber(  runNr   ) ;
  //  evt->setEventNumber( 0 ) ;
  //  evt->setDetectorName( detName ) ;
  
  // create the collection we want to store the data in 
  LCCollectionVec* mappingCol = new LCCollectionVec( LCIO::LCGENERICOBJECT )  ;

  // open the input file
  filebuf infile;
  infile.open (argv[1],ios::in);
  istream inputstream(&infile);

  int linenumber = -1;
  // read all lines in the file
  while (!inputstream.eof() )
  {
    // read the next line from the file
    std::string line;
    std::getline(inputstream, line);

    if (inputstream.eof()) //eof is reached
    {
	if (!line.empty())
	{
	  cout << "Ignoring non-empty line at end of file: " << line << endl;
	}
	break;
    }

    // the line as a stream so ew can stream into the variables
    std::stringstream linestream(line);
    linenumber++;
    
    // check if the next line is a comment
    if (linestream.peek() == '#')
    {
      cout << "DEBUG: skipping comment line " << linenumber << " : "  << line <<  std::endl;
      continue;
    }

    int hw_group, hw_channel, moduleID, row, padnr_in_row;

    // stream the data into the variables 
    linestream >>  hw_group >> hw_channel >> moduleID >> row >>  padnr_in_row;

    cout << "DEBUG:  reading line " << linenumber << " : "<< line << std::endl;      

    // test if everything went fine
    if (linestream.fail())
    {
      cout << "error reading line " << linenumber << " : "<< line << std::endl;      
      continue;
    } 
    
    //std::cout << "DEBUG: retrieving module "<<moduleID << std::endl;
    gear::TPCModule const & module = tpcParams.getModule( moduleID );

    //std::cout << "DEBUG: got module with id "<< module.getModuleID() << " at "<< &module << std::endl;

    int pad_index = module.getPadIndex(row, padnr_in_row);
    ADCChannelMapping * channelmapping =
      new  ADCChannelMapping(hw_channel,  pad_index , -1, moduleID, hw_group);
   
    mappingCol->addElement( channelmapping );
    
  }//  while (!inputstream.eof() )
  
  // add the mapping collection to the event
  evt->addCollection( mappingCol , "TPCChannelMapping" ) ;

  // write the event to the file
  lcWrt->writeEvent( evt ) ;
	
  // dump the event to the screen 
  LCTOOLS::dumpEvent( evt ) ;

  // ------------ IMPORTANT ------------- !
  // we created the event so we need to delete it ...
  delete evt ;
  // -------------------------------------

  lcWrt->close() ;
  
  // close the input text file
  infile.close();
     
  return 0;

}
