#ifndef ADD_EMPTY_PEDESTALS_PROCESSOR_H
#define ADD_EMPTY_PEDESTALS_PROCESSOR_H

#include "marlin/Processor.h"
#include "lcio.h"

//C++
#include <string>


/** Adds a collection of empty pedestals (pedestal and width set to 0)
 *  of all pads defined by GEAR.
 *
 *  The collection is added to the first event. A LCConditionsMgr is registered so the event
 *  is available in all further events. If you want to use the pedestal collection when reading
 *  from the output file you have to register a data file handler in the ConditionsProcessor.
 *
 *  @param PedestalCollectionName Name of the output collection with tpcconddata::Pedestal objects, default: TPCPedestals
 * 
 * @author Martin Killenberg, CERN
 *
 */

class AddEmptyPedestalsProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new AddEmptyPedestalsProcessor; }
  
  AddEmptyPedestalsProcessor();
  
  virtual ~AddEmptyPedestalsProcessor();

  virtual void init();
  
  virtual void processRunHeader(EVENT::LCRunHeader* run);
  
  virtual void processEvent(EVENT::LCEvent* evt);
  
  virtual void check(EVENT::LCEvent* evt);
  
  virtual void end();
  
  
 protected:

  std::string _pedestalColName; // name of the output collection
  bool _isFirstEventInRun; // flag whether this is the first event in a run
};
#endif // ADD_EMPTY_PEDESTALS_PROCESSOR_H
