#ifndef GenericADCElectronicsParameters_h
#define GenericADCElectronicsParameters_h 1

//C++
#include <iostream>
#include <string>
#include <utility>

//LCIO
#include "lcio.h"
#include "UTIL/LCFixedObject.h"

                                 // DAQ channel number and readout module ID here called channelID and readoutGroup
#define GenericADCElectronicsParametersNINTVals 3
                                 // GenericADCElectronicsParameters Value and GenericADCElectronicsParameters Width
#define GenericADCElectronicsParametersNFLOATVals 1
#define GenericADCElectronicsParametersNDOUBLEVals 0

namespace tpcconddata
{

    class GenericADCElectronicsParameters;

    typedef GenericADCElectronicsParameters LCGenericADCElectronicsParameters;

    /** Class that combines information on parameters and settings if the read-out electronics for storage.
     *  The readout group (module) number can be stored as integer.
     *  The polarity of the electronics is stored in an integer that should be +1 (positive polarity) or
     *  -1 (negative polarity).
     *  The precision of the read-out electronics can be stored in an integer variable that holds the maximum
     *  possible ADC counts (i.e. maxADCValue=255).
     *  The read-out frequency is store as a float variable in the unit MHz.
     *  This object is based on the LCFixedObject template.
     * @author R. Diener (DESY)
     */

    class GenericADCElectronicsParameters : public UTIL::LCFixedObject<GenericADCElectronicsParametersNINTVals,GenericADCElectronicsParametersNFLOATVals,GenericADCElectronicsParametersNDOUBLEVals>
    {

        public:

            /** Convenient constructor setting all values
             */
            GenericADCElectronicsParameters(int readoutGroupID, int polarity, int maxADCvalue, float frequency);

            /** 'Copy constructor' needed to interpret LCCollection read from file/database.
             */
            GenericADCElectronicsParameters(EVENT::LCObject* obj):UTIL::LCFixedObject<GenericADCElectronicsParametersNINTVals,
                GenericADCElectronicsParametersNFLOATVals,
                GenericADCElectronicsParametersNDOUBLEVals>(obj) { }

            /** Important for memory handling*/
            virtual ~GenericADCElectronicsParameters();

            // the class interface:

            // set the module number
            void setReadoutGroup(int readoutGroupID);
            // set the polarity value, should be -1 or +1
            void setPolarity(int polarity);
            // set the precision value of the electronics (maximum possible ADC value)
            void setMaxADCValue(int maxADCValue);
            // set the read out frequency of the read-out electronics (in MHz)
            void setFrequency(float frequency);
            // get the module number
            int getReadoutGroup() const;
            // get the polarity value, should be -1 or +1
            int getPolarity() const;
            // get the precision value of the electronics (maximum possible ADC value)
            int getMaxADCValue() const;
            // get the read out frequency of the read-out electronics (in MHz)
            float getFrequency() const;

            static std::string getRevision();
            static std::string getDefaultColName();

            void print(  std::ostream& os = std::cout ) const;

            // -------- need to implement abstract methods from LCGenericObject
            const std::string getTypeName() const
            {
                return std::string("GenericADCElectronicsParameters");
            }
            const std::string getDataDescription() const
            {
                return std::string("i:ReadoutGroup,Polarity,MaxADCValue,f:Frequency(MHz)");
            }

        protected:

    };                           // end of class

    std::ostream &operator<<(std::ostream &os, const GenericADCElectronicsParameters &p);

}                                //end namespace tpcconddata
#endif
