#ifndef TPCCONDDATA_EXCEPTION_H
#define TPCCONDDATA_EXCEPTION_H 1

#include <string>
#include <exception>

// define some exceptions similar to the ones used in hep.lcd.io
// the exceptions should be part of the corresponding namespace

namespace tpcconddata
{

    /**Exception class for MarlinTPC conditions.
     * @author R. Diener (DESY)
     */

    class Exception : public std::exception
    {

        protected:
            std::string message;

            Exception(){  /*no_op*/; };

        public:
            virtual ~Exception() throw() { /*no_op*/; };

            Exception( const std::string& text )
            {
                message = "tpcconddata::Exception: " + text ;
            };

            virtual const char* what() const  throw() { return  message.c_str(); };

    };

    /**ConditionObjectException used for errors accessing the event data.
     * @author R. Diener (DESY)
     */
    class ConditionsObjectException : public Exception
    {

        protected:
            ConditionsObjectException() {  /*no_op*/; };
        public:
            virtual ~ConditionsObjectException() throw() { /*no_op*/; };

            ConditionsObjectException( std::string text )
            {
                message = "tpcconddata::ConditionsObjectException: " + text;
            }
    };

    /**ConditionsObjectException used for data not available.
     * @author R. Diener (DESY)
     */
    class CondDataNotAvailableException : public ConditionsObjectException
    {

        public:
            virtual ~CondDataNotAvailableException() throw() { /*no_op*/; };

            CondDataNotAvailableException( std::string text )
            {
                message = "tpcconditions::CondDataNotAvailableException: " + text;
            };
    };

    /**ConditionsObjectException used for wrong object.
     * @author R. Diener (DESY)
     */
    class CondDataWrongObjectException : public ConditionsObjectException
    {

        public:
            virtual ~CondDataWrongObjectException() throw() { /*no_op*/; };

            CondDataWrongObjectException( std::string text )
            {
                message = "tpcconditions::CondDataWrongObjectException: " + text;
            };
    };

}                                // namespace
#endif                           /* ifndef TPCCONDDATA_EXCEPTION_H */
