#include "DumpHitsProcessor.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/TrackerHit.h>

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

DumpHitsProcessor aDumpHitsProcessor;
    
DumpHitsProcessor::DumpHitsProcessor(): Processor("DumpHitsProcessor")
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACKERHIT,
				"InputHitsCollection",
				"Name of the input hits collection, default: TPCHits",
				_inputColName ,
				std::string("TPCHits") ) ;
    // modify processor description
    _description = " Dump the coordinates of all hits in the input collection." ;
}
    
void DumpHitsProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data
    cout << "# Event: " << evt->getEventNumber() << endl;

    // we need a LCCollectionVec to use iterators, as LCCollectios is o not a std::vector
    LCCollectionVec* InputCollection=0;
    try{
	InputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	// no input data
	// just continue ...
	return;
    }


    if( InputCollection != 0 )
    {
	cout << "# hitNumber \t x \t y \t z" << endl;
	int hitNr=0;
	for(LCCollectionVec::iterator hitIter = InputCollection->begin() ; 
	    hitIter < InputCollection->end(); hitIter++ )
	{
	    TrackerHit *hit = dynamic_cast<TrackerHit *>(*hitIter);
	    if (hit==0)
	    {
		throw DataNotAvailableException("DumpHitsProcessor: Input collection does not contain TrackerHits");
	    }

	    const double *pos = hit->getPosition();
	    cout << hitNr++<<"\t "<<pos[0]<<"\t "
		 <<pos[1]<<"\t "<<pos[2]<< endl;
	}
    }// if( InputCollection != 0 )

}// processEvent


}// namespace 
